#include "qregexp.h"
#include "qptrlist.h"

#include "albumitem.h"
#include "trackitem.h"

AlbumItem::AlbumItem(QListViewItem *parent, QString artist, QString album, QString url) 
  : QListViewItem(parent, album),
    hasdata(false), findingdata(false), downloadatdone(false), signalemitter(0), 
    getter(url), artist(artist), album(album)
{
  setExpandable(true);
  connect(&getter, SIGNAL(transferDone(QString*)), this, SLOT(slotTransferDone(QString*)));
}
  
void AlbumItem::setOpen(bool o)
{
  if (!(hasdata || findingdata)) {
    findingdata = true;
    getter.get();
  }

  QListViewItem::setOpen(o);
}

void AlbumItem::initDownloads(SearchView *fromhere)
{
  if (hasdata) {
    QPtrListIterator<TrackItem> it(tracks);
    
    for (;it.current(); ++it) {
      it.current()->initDownloads(fromhere);
    }
    return;
  }

  if (!findingdata) {
    getter.get();
    findingdata = true;
  }

  downloadatdone = true;
  signalemitter = fromhere;
}

void AlbumItem::slotTransferDone(QString *str)
{
  int pos = 0;
  QRegExp length("total time:[\\s]*([\\d]*:[\\d]*)<br>");
  QRegExp track("<tr><td valign=top>[\\s]*([\\d]*).</td><td valign=top>[\\s]*([\\d]*:[\\d]*)</td><td><b>([^<]*)</b>"); 

  pos = length.search(*str);
  if (pos != -1) setText(1, length.cap(1));
  while ( (pos = track.search(*str, pos)) != -1 ) {
    QString tracknum = track.cap(1);
    QString title = track.cap(3);
    QString length = track.cap(2);
    TrackItem *titem = new TrackItem(this, tracknum.toInt(), artist, album, title, length);
    tracks.append(titem);
    if (downloadatdone) titem->initDownloads(signalemitter);
    pos += track.matchedLength();
  }
  hasdata = true;
}
