#ifndef CONFIG_H
#define CONFIG_H

#include "qobject.h"
#include "qsettings.h"
#include "qtimer.h"
#include "qstring.h"
#include "qstringlist.h"

#define SETTINGSWRITE 3000

class SettingsWriter : public QObject
{
Q_OBJECT

public:
  SettingsWriter() : timeout(new QTimer()), s(0) 
  {
    connect(timeout,SIGNAL(timeout()),this,SLOT(writeSettings()));
  }

  ~SettingsWriter() {
    delete s;
    delete timeout;
  }

  QSettings *getSettings() 
  {
    timeout->start(SETTINGSWRITE,true);
    if (!s) s = new QSettings();
    return s;
  }

public slots:
  void writeSettings()
  {
    delete s;
    s = 0;
  }
private:
  QTimer *timeout;
  QSettings *s;
};

enum HubsProxyEnum  {
  NoProxy, SOCKSProxy, WebProxy
};

enum ProxyVerEnum {
  SOCKS4, SOCKS5
};

class Config
{
public:
  QString name;

  QString downloaddir;
  QString fileformat;
  bool useid3v2;
  bool setid3;
  bool rejectid3;
  QString tempdir;
  QString tempprefix;

  bool viewdetails;

  bool reqartist;
  bool reqalbum;
  QString matchchars;
  int minbps;
  int maxbps;

  int delay;  
  int dlcand;
  int canddelay;
  int candtimeout;
  int candtryout;
  int maxqueue;
  int maxgdlqueue;
  int nosourcetimeout;
  int stalledtimeout;
  int searchtimeout;
  int gdlnosourcetimeout;
  int gdlstalltimeout;
  int gdlsourcestalltimeout;
  int gdlsourceattempts;
  
  /* Connection options */
  bool activemode;

  bool useoverrideip;
  QString overrideip;
  bool useoverrideport;
  int overrideport;
  bool dynamicip;

  bool useproxy;
  ProxyVerEnum proxyversion;
  QString proxyaddr;
  int proxyport;
  QString proxyuser;
  QString proxypass;

  HubsProxyEnum hubsproxytype;
  QString hubsproxyaddr;
  int hubsproxyport;

  QValueList<int> activehubs;
};

class DcHubConfig
{
public:
  QString name;
  QString hubaddr;
  QString nick;
  QString desc;
  QString email;
  QString conn;
  QStringList shareddirs;
  bool usesizeoffs;
  double sizeoffs;
  bool usevshare;
  QString vsharedir;
  int maxupl;
  bool useullim;
  int ullim;
};

class AudioTransfer
{
public:
  AudioTransfer() : trackno(0), length(0) {}
  AudioTransfer(QString artist, QString album, QString title,
		int trackno, int length) :
    artist(artist), album(album), title(title),
    trackno(trackno), length(length)
    {}

  QString artist;
  QString album;
  QString title;
  int trackno;
  int length;
};

void saveTransfer(AudioTransfer trans);
void removeTransfer(AudioTransfer trans);
QValueList<AudioTransfer> retrieveTransfers();

void configSetup();
bool readConfig();
void writeConfig();

int newConfig();
void killConfig(int id);
Config *getConfig(int id);

bool setActiveConfig(int id);
int getActiveConfig();

QValueList<int> listConfigs();

int newHub();
void killHub(int id);
DcHubConfig *getHub(int id);
QValueList<int> listHubs();

extern Config *global_config;
inline Config *activeConfig() { return global_config; }
  
#endif // CONFIG_H
