#ifndef DOWNLOADVIEW_H
#define DOWNLOADVIEW_H

#include "qlistview.h"
#include "paintlistviewitem.h"
#include "paintprogress.h"

#include "transfer.h"
#include "transferagent.h"

class DownloadView;
class DownloadItem : public PaintListViewItem, public TransferAgent
{
public:
  DownloadItem(DownloadView *parent, bool newtransfer = false);
  DownloadItem(DownloadItem *parent, bool newtransfer = false);
  virtual ~DownloadItem();

  /* TransferAgent methods */
  virtual void transferCreated(Transfer *transfer, QString id);
  virtual void transferStatusChanged(QString status = 0, long pos = -1, long size = -1,
				     long speed = -1, long eta = -1,
				     double progress = -1, double target = -1);
  virtual void transferCompleted();
  virtual void transferRemoved();
  virtual TransferAgent *transferChildCreated();
  virtual void startRename(int col); 
  /* Called from DownloadView */
  void doPopupMenu(const QPoint &pos);
protected:
  static QString formatSize(long size);
  static QString formatSpeed(long speed);
  static QString formatPercentage(double frac);
  static QString formatTime(long sec);
  void setProgress(double p);
  void setDualProgress(double p1, double p2);

  virtual void okRename(int col);
  virtual void cancelRename(int col);
private:
  PaintProgress *progressbar;
  QString key;
  bool completed;
  bool createOnRename;
};

class DownloadView : public QListView, public TransferAgent
{
Q_OBJECT

public:
  DownloadView(QWidget *parent = 0);
  virtual ~DownloadView();

  /* TransferAgent methods */
  virtual void transferCreated(Transfer *transfer, QString id);
  virtual void transferStatusChanged(QString status = 0, long pos = -1, long size = -1,
				     long speed = -1, long eta = -1,
				     double progress = -1, double target = -1);
  virtual void transferCompleted();
  virtual void transferRemoved();
  virtual TransferAgent *transferChildCreated();

  void getNoupdateLock() { noupdatelock++; }
  void dropNoupdateLock() { noupdatelock--; }
  bool mayUpdate() { return !noupdatelock; }
private slots:
  void doPopupMenu(QListViewItem *item, const QPoint &pos, int col);
private:
  int noupdatelock; 
};

#endif // DOWNLOADVIEW_H
