#include "qlistview.h"
#include "qregexp.h"

#include "hublistgetter.h"
#include "config.h"

class PublicHubListItem : public QListViewItem 
{
public:
  PublicHubListItem(QListView *parent, QString name,
		    QString users, QString desc, QString addr) :
    QListViewItem(parent, name, users, desc, addr)
  {
  }
  
  int PublicHubListItem::compare(QListViewItem *i, int col, bool ascending) const
  {
    if (col != 1) return QListViewItem::compare(i, col, ascending);
    return text(1).toLong() - i->text(1).toLong();
  }
};

HubListGetter::HubListGetter(QListView *view) : 
  QProcess(QString(HUBLISTBINARY)), view(view)
{
  connect(this, SIGNAL(processExited()), this, SLOT(processDied()));
  connect(this, SIGNAL(readyReadStdout()), this, SLOT(slotReadyRead()));
}

HubListGetter::~HubListGetter()
{
}

void HubListGetter::getHubs()
{
  if (isRunning()) kill();

  clearArguments();
  addArgument(QString(HUBLISTBINARY));
  switch (global_config->hubsproxytype) {
  case NoProxy: break;
  case WebProxy: 
    addArgument("--proxy");
    addArgument(global_config->hubsproxyaddr);
    addArgument(QString::number(global_config->hubsproxyport));
    break;
  case SOCKSProxy: 
    addArgument("--socks");
    addArgument(global_config->proxyaddr);
    addArgument(QString::number(global_config->proxyport));
    if (!global_config->proxyuser.isEmpty())
      addArgument(global_config->proxyuser);
    break;
  }

  gotdata = false;
  view->clear();
  new QListViewItem(view, "Getting hub list...");
  partofline = "";

  if (!start()) processDied();
}

void HubListGetter::processDied()
{
  if (!gotdata) {
    view->clear();
    new QListViewItem(view, "Error when getting hub list!");
  }
}

void HubListGetter::slotReadyRead()
{
  while (canReadLineStdout()) {
    QString tmp = partofline+readLineStdout();
    QRegExp hubrx("([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|([^\\|]*)\\|");
    if (hubrx.search(tmp) != -1) {
      if (!gotdata) view->clear();
      new PublicHubListItem(view, hubrx.cap(1), hubrx.cap(4), hubrx.cap(3), hubrx.cap(2));
      gotdata = true;
    }
    partofline = "";
  }
  partofline += QString(readStdout());
}
