#include "qtextedit.h"

#include "hubstatusview.h"

static QString formatSize(double size)
{
  QString str;
  
  if (size < 0) str = "";
  else if (size >= 1.0*1024*1024*1024*1024*1024*10)
    str.sprintf("%.1fPB", size / (1.0*1024*1024*1024*1024*1024));
  else if (size >= 1.0*1024*1024*1024*1024*1024)
    str.sprintf("%.2fPB", size / (1.0*1024*1024*1024*1024*1024));
  else if (size >= 1.0*1024*1024*1024*1024*10)
    str.sprintf("%.1fTB", size / (1.0*1024*1024*1024*1024));
  else if (size >= 1.0*1024*1024*1024*1024)
    str.sprintf("%.2fTB", size / (1.0*1024*1024*1024*1024));
  else if (size >= 1.0*1024*1024*1024*10)
    str.sprintf("%.1fGB", size / (1.0*1024*1024*1024));
  else if (size >= 1.0*1024*1024*1024)
    str.sprintf("%.2fGB", size / (1.0*1024*1024*1024));
  else if (size >= 1.0*1024*1024*10)
    str.sprintf("%.1fMB", size / (1.0*1024*1024));
  else if (size >= 1.0*1024*1024)
    str.sprintf("%.2fMB", size / (1.0*1024*1024));
  else if (size >= 1.0*1024*10)
    str.sprintf("%.1fKB", size / (1.0*1024));
  else if (size >= 1.0*1024)
    str.sprintf("%.2fKB", size / (1.0*1024));
  else if (size >= 1.0*10)
    str.sprintf("%.1fB", size * 1.0);
  else 
    str.sprintf("%.2fB", size * 1.0);

  return str;
}

HubStatusItem::HubStatusItem(QTextEdit *logview, QListView *parent) : 
  QListViewItem(parent), logview(logview)
{
}

void HubStatusItem::activate()
{
  logview->setText(logtext.join("\n"));
  logview->scrollToBottom();
}

void HubStatusItem::update(QString adress, QString name, QString status, int users, int downloads, int uploads, double shares)
{
  setText(0,adress);
  setText(1,name);
  setText(2,status);
  setText(3,QString::number(users));
  setText(4,QString::number(downloads));
  setText(5,QString::number(uploads));
  setText(6,formatSize(shares));
}

void HubStatusItem::logText(QString newlogtext)
{
  if (logtext.count() >= MAXLOGLINES) logtext.pop_front();
  logtext.push_back(newlogtext);
  
  if (isSelected()) {
    if (logview->paragraphs() >= HARDMAXLOGLINES)
      logview->setText(logtext.join("\n"));
    else
      logview->append(newlogtext);
  }
}

HubStatusView::HubStatusView(QWidget *parent) : HubStatusViewUI(parent)
{
  hubs.setAutoDelete(true);
  hubsOutputEdit->setReadOnly(true);
  hubsOutputEdit->setTextFormat(QTextEdit::PlainText);
  /*  hubsOutputEdit->setTextFormat(QTextEdit::LogText);
      hubsOutputEdit->setWordWrap(QTextEdit::WidgetWidth); */
}

void HubStatusView::addHub(int hub)
{
  hubs.insert(hub, new HubStatusItem(hubsOutputEdit, hubsStatusList));
}

void HubStatusView::killHub(int hub)
{
  HubStatusItem *item = hubs.find(hub);
  if (item && item->isSelected()) {
    hubs.remove(hub);
    if (hubsStatusList->childCount())
      hubsStatusList->firstChild()->setSelected(true);
    else
      hubsOutputEdit->clear();
  }
  else hubs.remove(hub);
}

void HubStatusView::updateHub(int hubid, QString adress, QString name, QString status, int users, int downloads, int uploads, double shares)
{
  HubStatusItem *hub = hubs.find(hubid);
  if (hub)
    hub->update(adress, name, status, users, downloads, uploads, shares);
}

void HubStatusView::logText(int hub, QString logtext)
{
  HubStatusItem *item = hubs.find(hub);
  if (item) item->logText(logtext);
}
